// Generated by the protocol buffer compiler.  DO NOT EDIT!
// NO CHECKED-IN PROTOBUF GENCODE
// source: bloaty.proto
// Protobuf C++ Version: 7.35.1

#ifndef bloaty_2eproto_2epb_2eh
#define bloaty_2eproto_2epb_2eh

#include <limits>
#include <string>
#include <type_traits>
#include <utility>

// clang-format off
#include "google/protobuf/runtime_version.h"
#if PROTOBUF_VERSION != 7035001
#error "Protobuf C++ gencode is built with an incompatible version of"
#error "Protobuf C++ headers/runtime. See"
#error "https://protobuf.dev/support/cross-version-runtime-guarantee/#cpp"
#endif
#include "google/protobuf/io/coded_stream.h"
#include "google/protobuf/arena.h"
#include "google/protobuf/arenastring.h"
#include "google/protobuf/generated_message_tctable_decl.h"
#include "google/protobuf/generated_message_util.h"
#include "google/protobuf/metadata_lite.h"
#include "google/protobuf/generated_message_reflection.h"
#include "google/protobuf/message.h"
#include "google/protobuf/message_lite.h"
#include "google/protobuf/repeated_field.h"  // IWYU pragma: export
#include "google/protobuf/extension_set.h"  // IWYU pragma: export
#include "google/protobuf/generated_enum_reflection.h"
#include "google/protobuf/unknown_field_set.h"
// @@protoc_insertion_point(includes)

// Must be included last.
#include "google/protobuf/port_def.inc"

#define PROTOBUF_INTERNAL_EXPORT_bloaty_2eproto

namespace google {
namespace protobuf {
namespace internal {
template <typename T>
::absl::string_view GetAnyMessageName();
}  // namespace internal
}  // namespace protobuf
}  // namespace google

// Internal implementation detail -- do not use these members.
struct TableStruct_bloaty_2eproto {
  static const ::uint32_t offsets[];
};
extern "C" {
extern const ::google::protobuf::internal::DescriptorTable descriptor_table_bloaty_2eproto;
}  // extern "C"
namespace bloaty {
enum Options_Demangle : int;
extern const uint32_t Options_Demangle_internal_data_[];
enum Options_SortBy : int;
extern const uint32_t Options_SortBy_internal_data_[];
class CustomDataSource;
struct CustomDataSourceGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern CustomDataSourceGlobalsTypeInternal CustomDataSource_globals_;
extern const ::google::protobuf::internal::ClassDataFull CustomDataSource_class_data_;
#else
extern const CustomDataSourceGlobalsTypeInternal CustomDataSource_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class Options;
struct OptionsGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern OptionsGlobalsTypeInternal Options_globals_;
extern const ::google::protobuf::internal::ClassDataFull Options_class_data_;
#else
extern const OptionsGlobalsTypeInternal Options_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class Regex;
struct RegexGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern RegexGlobalsTypeInternal Regex_globals_;
extern const ::google::protobuf::internal::ClassDataFull Regex_class_data_;
#else
extern const RegexGlobalsTypeInternal Regex_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
}  // namespace bloaty
namespace google {
namespace protobuf {
template <>
internal::EnumTraitsT<::bloaty::Options_Demangle_internal_data_>
    internal::EnumTraitsImpl::value<::bloaty::Options_Demangle>;
template <>
internal::EnumTraitsT<::bloaty::Options_SortBy_internal_data_>
    internal::EnumTraitsImpl::value<::bloaty::Options_SortBy>;
}  // namespace protobuf
}  // namespace google

namespace bloaty {
enum Options_Demangle : int {
  Options_Demangle_DEMANGLE_SHORT = 0,
  Options_Demangle_DEMANGLE_FULL = 1,
  Options_Demangle_DEMANGLE_NONE = 2,
};

extern const uint32_t Options_Demangle_internal_data_[];
inline constexpr Options_Demangle Options_Demangle_Demangle_MIN =
    static_cast<Options_Demangle>(0);
inline constexpr Options_Demangle Options_Demangle_Demangle_MAX =
    static_cast<Options_Demangle>(2);
[[nodiscard]] inline bool Options_Demangle_IsValid(int value) {
  return 0 <= value && value <= 2;
}
inline constexpr int Options_Demangle_Demangle_ARRAYSIZE = 2 + 1;
[[nodiscard]] const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL
Options_Demangle_descriptor();
[[nodiscard]] inline auto ProtobufInternalGetEnumDescriptor(Options_Demangle) {
  return Options_Demangle_descriptor();
}
template <typename T>
[[nodiscard]] const ::std::string& Options_Demangle_Name(T value) {
  static_assert(::std::is_same<T, Options_Demangle>::value ||
                    ::std::is_integral<T>::value,
                "Incorrect type passed to Demangle_Name().");
  return Options_Demangle_Name(static_cast<Options_Demangle>(value));
}
template <>
[[nodiscard]] inline const ::std::string& Options_Demangle_Name(Options_Demangle value) {
  return ::google::protobuf::internal::NameOfDenseEnum<Options_Demangle_descriptor, 0, 2>(
      static_cast<int>(value));
}
[[nodiscard]] inline bool Options_Demangle_Parse(
    ::absl::string_view name, Options_Demangle* PROTOBUF_NONNULL value) {
  return ::google::protobuf::internal::ParseNamedEnum<Options_Demangle>(Options_Demangle_descriptor(), name,
                                           value);
}
enum Options_SortBy : int {
  Options_SortBy_SORTBY_BOTH = 0,
  Options_SortBy_SORTBY_VMSIZE = 1,
  Options_SortBy_SORTBY_FILESIZE = 2,
};

extern const uint32_t Options_SortBy_internal_data_[];
inline constexpr Options_SortBy Options_SortBy_SortBy_MIN =
    static_cast<Options_SortBy>(0);
inline constexpr Options_SortBy Options_SortBy_SortBy_MAX =
    static_cast<Options_SortBy>(2);
[[nodiscard]] inline bool Options_SortBy_IsValid(int value) {
  return 0 <= value && value <= 2;
}
inline constexpr int Options_SortBy_SortBy_ARRAYSIZE = 2 + 1;
[[nodiscard]] const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL
Options_SortBy_descriptor();
[[nodiscard]] inline auto ProtobufInternalGetEnumDescriptor(Options_SortBy) {
  return Options_SortBy_descriptor();
}
template <typename T>
[[nodiscard]] const ::std::string& Options_SortBy_Name(T value) {
  static_assert(::std::is_same<T, Options_SortBy>::value ||
                    ::std::is_integral<T>::value,
                "Incorrect type passed to SortBy_Name().");
  return Options_SortBy_Name(static_cast<Options_SortBy>(value));
}
template <>
[[nodiscard]] inline const ::std::string& Options_SortBy_Name(Options_SortBy value) {
  return ::google::protobuf::internal::NameOfDenseEnum<Options_SortBy_descriptor, 0, 2>(
      static_cast<int>(value));
}
[[nodiscard]] inline bool Options_SortBy_Parse(
    ::absl::string_view name, Options_SortBy* PROTOBUF_NONNULL value) {
  return ::google::protobuf::internal::ParseNamedEnum<Options_SortBy>(Options_SortBy_descriptor(), name,
                                           value);
}
using ::google::protobuf::internal::generated_enum::AbslParseFlag;
using ::google::protobuf::internal::generated_enum::AbslUnparseFlag;

// ===================================================================


// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED Regex final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:bloaty.Regex) */ {
 public:
  inline Regex() : Regex(nullptr) {}
  ~Regex() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(Regex* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(Regex));
  }
#endif

  template <typename = void>
  explicit constexpr Regex(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline Regex(const Regex& from) : Regex(nullptr, from) {}
  inline Regex(Regex&& from) noexcept : Regex(nullptr, ::std::move(from)) {}
  inline Regex& operator=(const Regex& from) {
    CopyFrom(from);
    return *this;
  }
  inline Regex& operator=(Regex&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const Regex& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<Regex>(&Regex_globals_);
  }
  static constexpr int kIndexInFileMessages = 2;
  friend void swap(Regex& a, Regex& b) { a.Swap(&b); }
  inline void Swap(Regex* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(Regex* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] Regex* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<Regex>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const Regex& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const Regex& from) { Regex::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(Regex* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "bloaty.Regex"; }

  explicit Regex(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  Regex(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Regex& from);
  Regex(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, Regex&& from) noexcept
      : Regex(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kPatternFieldNumber = 1,
    kReplacementFieldNumber = 2,
  };
  // optional string pattern = 1;
  [[nodiscard]] bool has_pattern()
      const;
  void clear_pattern() ;
  [[nodiscard]] const ::std::string& pattern() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_pattern(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_pattern();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_pattern();
  void set_allocated_pattern(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_pattern() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_pattern(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_pattern();

  public:
  // optional string replacement = 2;
  [[nodiscard]] bool has_replacement()
      const;
  void clear_replacement() ;
  [[nodiscard]] const ::std::string& replacement() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_replacement(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_replacement();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_replacement();
  void set_allocated_replacement(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_replacement() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_replacement(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_replacement();

  public:
  // @@protoc_insertion_point(class_scope:bloaty.Regex)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<1, 2,
                          0, 0,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const Regex& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::internal::ArenaStringPtr pattern_;
    ::google::protobuf::internal::ArenaStringPtr replacement_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_bloaty_2eproto;
};
// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED CustomDataSource final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:bloaty.CustomDataSource) */ {
 public:
  inline CustomDataSource() : CustomDataSource(nullptr) {}
  ~CustomDataSource() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(CustomDataSource* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(CustomDataSource));
  }
#endif

  template <typename = void>
  explicit constexpr CustomDataSource(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline CustomDataSource(const CustomDataSource& from) : CustomDataSource(nullptr, from) {}
  inline CustomDataSource(CustomDataSource&& from) noexcept : CustomDataSource(nullptr, ::std::move(from)) {}
  inline CustomDataSource& operator=(const CustomDataSource& from) {
    CopyFrom(from);
    return *this;
  }
  inline CustomDataSource& operator=(CustomDataSource&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const CustomDataSource& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<CustomDataSource>(&CustomDataSource_globals_);
  }
  static constexpr int kIndexInFileMessages = 1;
  friend void swap(CustomDataSource& a, CustomDataSource& b) { a.Swap(&b); }
  inline void Swap(CustomDataSource* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(CustomDataSource* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] CustomDataSource* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<CustomDataSource>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const CustomDataSource& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const CustomDataSource& from) { CustomDataSource::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(CustomDataSource* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "bloaty.CustomDataSource"; }

  explicit CustomDataSource(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  CustomDataSource(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const CustomDataSource& from);
  CustomDataSource(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, CustomDataSource&& from) noexcept
      : CustomDataSource(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kRewriteFieldNumber = 3,
    kNameFieldNumber = 1,
    kBaseDataSourceFieldNumber = 2,
  };
  // repeated .bloaty.Regex rewrite = 3;
  [[nodiscard]] int rewrite_size()
      const;
  private:
  int _internal_rewrite_size() const;

  public:
  void clear_rewrite() ;
  [[nodiscard]] const ::bloaty::Regex& rewrite(int index) const;
  [[nodiscard]] ::bloaty::Regex* PROTOBUF_NONNULL mutable_rewrite(int index);
  ::bloaty::Regex* PROTOBUF_NONNULL add_rewrite();
  [[nodiscard]] const ::google::protobuf::RepeatedPtrField<::bloaty::Regex>&
  rewrite() const;
  [[nodiscard]] ::google::protobuf::RepeatedPtrField<::bloaty::Regex>* PROTOBUF_NONNULL
  mutable_rewrite();

  private:
  const ::google::protobuf::RepeatedPtrField<::bloaty::Regex>& _internal_rewrite() const;
  ::google::protobuf::RepeatedPtrField<::bloaty::Regex>* PROTOBUF_NONNULL _internal_mutable_rewrite();

  public:
  // optional string name = 1;
  [[nodiscard]] bool has_name()
      const;
  void clear_name() ;
  [[nodiscard]] const ::std::string& name() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_name(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_name();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_name();
  void set_allocated_name(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_name() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_name(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_name();

  public:
  // optional string base_data_source = 2;
  [[nodiscard]] bool has_base_data_source()
      const;
  void clear_base_data_source() ;
  [[nodiscard]] const ::std::string& base_data_source() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_base_data_source(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_base_data_source();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_base_data_source();
  void set_allocated_base_data_source(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_base_data_source() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_base_data_source(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_base_data_source();

  public:
  // @@protoc_insertion_point(class_scope:bloaty.CustomDataSource)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<2, 3,
                          1, 0,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const CustomDataSource& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::RepeatedPtrField< ::bloaty::Regex > rewrite_;
    ::google::protobuf::internal::ArenaStringPtr name_;
    ::google::protobuf::internal::ArenaStringPtr base_data_source_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_bloaty_2eproto;
};
// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED Options final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:bloaty.Options) */ {
 public:
  inline Options() : Options(nullptr) {}
  ~Options() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(Options* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(Options));
  }
#endif

  template <typename = void>
  explicit constexpr Options(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline Options(const Options& from) : Options(nullptr, from) {}
  inline Options(Options&& from) noexcept : Options(nullptr, ::std::move(from)) {}
  inline Options& operator=(const Options& from) {
    CopyFrom(from);
    return *this;
  }
  inline Options& operator=(Options&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const Options& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<Options>(&Options_globals_);
  }
  static constexpr int kIndexInFileMessages = 0;
  friend void swap(Options& a, Options& b) { a.Swap(&b); }
  inline void Swap(Options* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(Options* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] Options* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<Options>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const Options& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const Options& from) { Options::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(Options* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "bloaty.Options"; }

  explicit Options(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  Options(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Options& from);
  Options(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, Options&& from) noexcept
      : Options(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------
  using Demangle = Options_Demangle;
  static constexpr Demangle DEMANGLE_SHORT = Options_Demangle_DEMANGLE_SHORT;
  static constexpr Demangle DEMANGLE_FULL = Options_Demangle_DEMANGLE_FULL;
  static constexpr Demangle DEMANGLE_NONE = Options_Demangle_DEMANGLE_NONE;
  [[nodiscard]] static inline bool Demangle_IsValid(int value) {
    return Options_Demangle_IsValid(value);
  }
  static constexpr Demangle Demangle_MIN = Options_Demangle_Demangle_MIN;
  static constexpr Demangle Demangle_MAX = Options_Demangle_Demangle_MAX;
  static constexpr int Demangle_ARRAYSIZE = Options_Demangle_Demangle_ARRAYSIZE;
  static inline const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL Demangle_descriptor() {
    return Options_Demangle_descriptor();
  }
  template <typename T>
  [[nodiscard]] static inline const ::std::string& Demangle_Name(T value) {
    return Options_Demangle_Name(value);
  }
  [[nodiscard]] static inline bool Demangle_Parse(
      ::absl::string_view name, Demangle* PROTOBUF_NONNULL value) {
    return Options_Demangle_Parse(name, value);
  }
  using SortBy = Options_SortBy;
  static constexpr SortBy SORTBY_BOTH = Options_SortBy_SORTBY_BOTH;
  static constexpr SortBy SORTBY_VMSIZE = Options_SortBy_SORTBY_VMSIZE;
  static constexpr SortBy SORTBY_FILESIZE = Options_SortBy_SORTBY_FILESIZE;
  [[nodiscard]] static inline bool SortBy_IsValid(int value) {
    return Options_SortBy_IsValid(value);
  }
  static constexpr SortBy SortBy_MIN = Options_SortBy_SortBy_MIN;
  static constexpr SortBy SortBy_MAX = Options_SortBy_SortBy_MAX;
  static constexpr int SortBy_ARRAYSIZE = Options_SortBy_SortBy_ARRAYSIZE;
  static inline const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL SortBy_descriptor() {
    return Options_SortBy_descriptor();
  }
  template <typename T>
  [[nodiscard]] static inline const ::std::string& SortBy_Name(T value) {
    return Options_SortBy_Name(value);
  }
  [[nodiscard]] static inline bool SortBy_Parse(
      ::absl::string_view name, SortBy* PROTOBUF_NONNULL value) {
    return Options_SortBy_Parse(name, value);
  }

  // accessors -------------------------------------------------------
  enum : int {
    kFilenameFieldNumber = 1,
    kBaseFilenameFieldNumber = 2,
    kDataSourceFieldNumber = 3,
    kCustomDataSourceFieldNumber = 8,
    kDebugFilenameFieldNumber = 10,
    kDisassembleFunctionFieldNumber = 9,
    kSourceFilterFieldNumber = 13,
    kDemangleFieldNumber = 5,
    kSortByFieldNumber = 6,
    kVerboseLevelFieldNumber = 7,
    kDumpRawMapFieldNumber = 14,
    kDebugVmaddrFieldNumber = 11,
    kDebugFileoffFieldNumber = 12,
    kMaxRowsPerLevelFieldNumber = 4,
  };
  // repeated string filename = 1;
  [[nodiscard]] int filename_size()
      const;
  private:
  int _internal_filename_size() const;

  public:
  void clear_filename() ;
  [[nodiscard]] const ::std::string& filename(int index) const;
  ::std::string* PROTOBUF_NONNULL mutable_filename(int index);
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_filename(int index, Arg_&& value, Args_... args);
  ::std::string* PROTOBUF_NONNULL add_filename();
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void add_filename(Arg_&& value, Args_... args);
  [[nodiscard]] const
      ::google::protobuf::RepeatedPtrField<::std::string>&
      filename() const;
  [[nodiscard]] ::google::protobuf::RepeatedPtrField<::std::string>*
      PROTOBUF_NONNULL
      mutable_filename();

  private:
  const ::google::protobuf::RepeatedPtrField<::std::string>& _internal_filename() const;
  ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL _internal_mutable_filename();

  public:
  // repeated string base_filename = 2;
  [[nodiscard]] int base_filename_size()
      const;
  private:
  int _internal_base_filename_size() const;

  public:
  void clear_base_filename() ;
  [[nodiscard]] const ::std::string& base_filename(int index) const;
  ::std::string* PROTOBUF_NONNULL mutable_base_filename(int index);
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_base_filename(int index, Arg_&& value, Args_... args);
  ::std::string* PROTOBUF_NONNULL add_base_filename();
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void add_base_filename(Arg_&& value, Args_... args);
  [[nodiscard]] const
      ::google::protobuf::RepeatedPtrField<::std::string>&
      base_filename() const;
  [[nodiscard]] ::google::protobuf::RepeatedPtrField<::std::string>*
      PROTOBUF_NONNULL
      mutable_base_filename();

  private:
  const ::google::protobuf::RepeatedPtrField<::std::string>& _internal_base_filename() const;
  ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL _internal_mutable_base_filename();

  public:
  // repeated string data_source = 3;
  [[nodiscard]] int data_source_size()
      const;
  private:
  int _internal_data_source_size() const;

  public:
  void clear_data_source() ;
  [[nodiscard]] const ::std::string& data_source(int index) const;
  ::std::string* PROTOBUF_NONNULL mutable_data_source(int index);
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_data_source(int index, Arg_&& value, Args_... args);
  ::std::string* PROTOBUF_NONNULL add_data_source();
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void add_data_source(Arg_&& value, Args_... args);
  [[nodiscard]] const
      ::google::protobuf::RepeatedPtrField<::std::string>&
      data_source() const;
  [[nodiscard]] ::google::protobuf::RepeatedPtrField<::std::string>*
      PROTOBUF_NONNULL
      mutable_data_source();

  private:
  const ::google::protobuf::RepeatedPtrField<::std::string>& _internal_data_source() const;
  ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL _internal_mutable_data_source();

  public:
  // repeated .bloaty.CustomDataSource custom_data_source = 8;
  [[nodiscard]] int custom_data_source_size()
      const;
  private:
  int _internal_custom_data_source_size() const;

  public:
  void clear_custom_data_source() ;
  [[nodiscard]] const ::bloaty::CustomDataSource& custom_data_source(int index) const;
  [[nodiscard]] ::bloaty::CustomDataSource* PROTOBUF_NONNULL mutable_custom_data_source(int index);
  ::bloaty::CustomDataSource* PROTOBUF_NONNULL add_custom_data_source();
  [[nodiscard]] const ::google::protobuf::RepeatedPtrField<::bloaty::CustomDataSource>&
  custom_data_source() const;
  [[nodiscard]] ::google::protobuf::RepeatedPtrField<::bloaty::CustomDataSource>* PROTOBUF_NONNULL
  mutable_custom_data_source();

  private:
  const ::google::protobuf::RepeatedPtrField<::bloaty::CustomDataSource>& _internal_custom_data_source() const;
  ::google::protobuf::RepeatedPtrField<::bloaty::CustomDataSource>* PROTOBUF_NONNULL _internal_mutable_custom_data_source();

  public:
  // repeated string debug_filename = 10;
  [[nodiscard]] int debug_filename_size()
      const;
  private:
  int _internal_debug_filename_size() const;

  public:
  void clear_debug_filename() ;
  [[nodiscard]] const ::std::string& debug_filename(int index) const;
  ::std::string* PROTOBUF_NONNULL mutable_debug_filename(int index);
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_debug_filename(int index, Arg_&& value, Args_... args);
  ::std::string* PROTOBUF_NONNULL add_debug_filename();
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void add_debug_filename(Arg_&& value, Args_... args);
  [[nodiscard]] const
      ::google::protobuf::RepeatedPtrField<::std::string>&
      debug_filename() const;
  [[nodiscard]] ::google::protobuf::RepeatedPtrField<::std::string>*
      PROTOBUF_NONNULL
      mutable_debug_filename();

  private:
  const ::google::protobuf::RepeatedPtrField<::std::string>& _internal_debug_filename() const;
  ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL _internal_mutable_debug_filename();

  public:
  // optional string disassemble_function = 9;
  [[nodiscard]] bool has_disassemble_function()
      const;
  void clear_disassemble_function() ;
  [[nodiscard]] const ::std::string& disassemble_function() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_disassemble_function(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_disassemble_function();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_disassemble_function();
  void set_allocated_disassemble_function(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_disassemble_function() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_disassemble_function(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_disassemble_function();

  public:
  // optional string source_filter = 13;
  [[nodiscard]] bool has_source_filter()
      const;
  void clear_source_filter() ;
  [[nodiscard]] const ::std::string& source_filter() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_source_filter(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_source_filter();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_source_filter();
  void set_allocated_source_filter(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_source_filter() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_source_filter(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_source_filter();

  public:
  // optional .bloaty.Options.Demangle demangle = 5 [default = DEMANGLE_SHORT];
  [[nodiscard]] bool has_demangle()
      const;
  void clear_demangle() ;
  [[nodiscard]] ::bloaty::Options_Demangle demangle() const;
  void set_demangle(::bloaty::Options_Demangle value);

  private:
  ::bloaty::Options_Demangle _internal_demangle() const;
  void _internal_set_demangle(::bloaty::Options_Demangle value);

  public:
  // optional .bloaty.Options.SortBy sort_by = 6 [default = SORTBY_BOTH];
  [[nodiscard]] bool has_sort_by()
      const;
  void clear_sort_by() ;
  [[nodiscard]] ::bloaty::Options_SortBy sort_by() const;
  void set_sort_by(::bloaty::Options_SortBy value);

  private:
  ::bloaty::Options_SortBy _internal_sort_by() const;
  void _internal_set_sort_by(::bloaty::Options_SortBy value);

  public:
  // optional int32 verbose_level = 7;
  [[nodiscard]] bool has_verbose_level()
      const;
  void clear_verbose_level() ;
  [[nodiscard]] ::int32_t verbose_level() const;
  void set_verbose_level(::int32_t value);

  private:
  ::int32_t _internal_verbose_level() const;
  void _internal_set_verbose_level(::int32_t value);

  public:
  // optional bool dump_raw_map = 14;
  [[nodiscard]] bool has_dump_raw_map()
      const;
  void clear_dump_raw_map() ;
  [[nodiscard]] bool dump_raw_map() const;
  void set_dump_raw_map(bool value);

  private:
  bool _internal_dump_raw_map() const;
  void _internal_set_dump_raw_map(bool value);

  public:
  // optional uint64 debug_vmaddr = 11;
  [[nodiscard]] bool has_debug_vmaddr()
      const;
  void clear_debug_vmaddr() ;
  [[nodiscard]] ::uint64_t debug_vmaddr() const;
  void set_debug_vmaddr(::uint64_t value);

  private:
  ::uint64_t _internal_debug_vmaddr() const;
  void _internal_set_debug_vmaddr(::uint64_t value);

  public:
  // optional uint64 debug_fileoff = 12;
  [[nodiscard]] bool has_debug_fileoff()
      const;
  void clear_debug_fileoff() ;
  [[nodiscard]] ::uint64_t debug_fileoff() const;
  void set_debug_fileoff(::uint64_t value);

  private:
  ::uint64_t _internal_debug_fileoff() const;
  void _internal_set_debug_fileoff(::uint64_t value);

  public:
  // optional int64 max_rows_per_level = 4 [default = 20];
  [[nodiscard]] bool has_max_rows_per_level()
      const;
  void clear_max_rows_per_level() ;
  [[nodiscard]] ::int64_t max_rows_per_level() const;
  void set_max_rows_per_level(::int64_t value);

  private:
  ::int64_t _internal_max_rows_per_level() const;
  void _internal_set_max_rows_per_level(::int64_t value);

  public:
  // @@protoc_insertion_point(class_scope:bloaty.Options)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<4, 14,
                          3, 0,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const Options& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::RepeatedPtrField<::std::string> filename_;
    ::google::protobuf::RepeatedPtrField<::std::string> base_filename_;
    ::google::protobuf::RepeatedPtrField<::std::string> data_source_;
    ::google::protobuf::RepeatedPtrField< ::bloaty::CustomDataSource > custom_data_source_;
    ::google::protobuf::RepeatedPtrField<::std::string> debug_filename_;
    ::google::protobuf::internal::ArenaStringPtr disassemble_function_;
    ::google::protobuf::internal::ArenaStringPtr source_filter_;
    int demangle_;
    int sort_by_;
    ::int32_t verbose_level_;
    bool dump_raw_map_;
    ::uint64_t debug_vmaddr_;
    ::uint64_t debug_fileoff_;
    ::int64_t max_rows_per_level_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_bloaty_2eproto;
};

// ===================================================================




// ===================================================================


#ifdef __GNUC__
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wstrict-aliasing"
#endif  // __GNUC__
// -------------------------------------------------------------------

// Options

// repeated string filename = 1;
inline int Options::_internal_filename_size() const {
  return _internal_filename().size();
}
inline int Options::filename_size() const {
  return _internal_filename_size();
}
inline void Options::clear_filename() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.filename_.Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
inline ::std::string* PROTOBUF_NONNULL Options::add_filename()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::std::string* _s =
      _internal_mutable_filename()->InternalAddWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), GetArena());
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_add_mutable:bloaty.Options.filename)
  return _s;
}
inline const ::std::string& Options::filename(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:bloaty.Options.filename)
  return _internal_filename().Get(index);
}
inline ::std::string* PROTOBUF_NONNULL Options::mutable_filename(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:bloaty.Options.filename)
  return _internal_mutable_filename()->Mutable(index);
}
template <typename Arg_, typename... Args_>
inline void Options::set_filename(int index, Arg_&& value, Args_... args) {
  ::google::protobuf::internal::AssignToString(
      *_internal_mutable_filename()->Mutable(index),
      ::std::forward<Arg_>(value), args... );
  // @@protoc_insertion_point(field_set:bloaty.Options.filename)
}
template <typename Arg_, typename... Args_>
inline void Options::add_filename(Arg_&& value, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::google::protobuf::internal::AddToRepeatedPtrField(
      ::google::protobuf::MessageLite::internal_visibility(), GetArena(),
      *_internal_mutable_filename(), ::std::forward<Arg_>(value),
      args... );
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_add:bloaty.Options.filename)
}
inline const ::google::protobuf::RepeatedPtrField<::std::string>& Options::filename()
    const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:bloaty.Options.filename)
  return _internal_filename();
}
inline ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL
Options::mutable_filename() ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_mutable_list:bloaty.Options.filename)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_filename();
}
inline const ::google::protobuf::RepeatedPtrField<::std::string>&
Options::_internal_filename() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.filename_;
}
inline ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL
Options::_internal_mutable_filename() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.filename_;
}

// repeated string base_filename = 2;
inline int Options::_internal_base_filename_size() const {
  return _internal_base_filename().size();
}
inline int Options::base_filename_size() const {
  return _internal_base_filename_size();
}
inline void Options::clear_base_filename() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.base_filename_.Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
inline ::std::string* PROTOBUF_NONNULL Options::add_base_filename()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::std::string* _s =
      _internal_mutable_base_filename()->InternalAddWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), GetArena());
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  // @@protoc_insertion_point(field_add_mutable:bloaty.Options.base_filename)
  return _s;
}
inline const ::std::string& Options::base_filename(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:bloaty.Options.base_filename)
  return _internal_base_filename().Get(index);
}
inline ::std::string* PROTOBUF_NONNULL Options::mutable_base_filename(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:bloaty.Options.base_filename)
  return _internal_mutable_base_filename()->Mutable(index);
}
template <typename Arg_, typename... Args_>
inline void Options::set_base_filename(int index, Arg_&& value, Args_... args) {
  ::google::protobuf::internal::AssignToString(
      *_internal_mutable_base_filename()->Mutable(index),
      ::std::forward<Arg_>(value), args... );
  // @@protoc_insertion_point(field_set:bloaty.Options.base_filename)
}
template <typename Arg_, typename... Args_>
inline void Options::add_base_filename(Arg_&& value, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::google::protobuf::internal::AddToRepeatedPtrField(
      ::google::protobuf::MessageLite::internal_visibility(), GetArena(),
      *_internal_mutable_base_filename(), ::std::forward<Arg_>(value),
      args... );
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  // @@protoc_insertion_point(field_add:bloaty.Options.base_filename)
}
inline const ::google::protobuf::RepeatedPtrField<::std::string>& Options::base_filename()
    const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:bloaty.Options.base_filename)
  return _internal_base_filename();
}
inline ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL
Options::mutable_base_filename() ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  // @@protoc_insertion_point(field_mutable_list:bloaty.Options.base_filename)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_base_filename();
}
inline const ::google::protobuf::RepeatedPtrField<::std::string>&
Options::_internal_base_filename() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.base_filename_;
}
inline ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL
Options::_internal_mutable_base_filename() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.base_filename_;
}

// repeated string debug_filename = 10;
inline int Options::_internal_debug_filename_size() const {
  return _internal_debug_filename().size();
}
inline int Options::debug_filename_size() const {
  return _internal_debug_filename_size();
}
inline void Options::clear_debug_filename() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.debug_filename_.Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
}
inline ::std::string* PROTOBUF_NONNULL Options::add_debug_filename()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::std::string* _s =
      _internal_mutable_debug_filename()->InternalAddWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), GetArena());
  SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  // @@protoc_insertion_point(field_add_mutable:bloaty.Options.debug_filename)
  return _s;
}
inline const ::std::string& Options::debug_filename(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:bloaty.Options.debug_filename)
  return _internal_debug_filename().Get(index);
}
inline ::std::string* PROTOBUF_NONNULL Options::mutable_debug_filename(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:bloaty.Options.debug_filename)
  return _internal_mutable_debug_filename()->Mutable(index);
}
template <typename Arg_, typename... Args_>
inline void Options::set_debug_filename(int index, Arg_&& value, Args_... args) {
  ::google::protobuf::internal::AssignToString(
      *_internal_mutable_debug_filename()->Mutable(index),
      ::std::forward<Arg_>(value), args... );
  // @@protoc_insertion_point(field_set:bloaty.Options.debug_filename)
}
template <typename Arg_, typename... Args_>
inline void Options::add_debug_filename(Arg_&& value, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::google::protobuf::internal::AddToRepeatedPtrField(
      ::google::protobuf::MessageLite::internal_visibility(), GetArena(),
      *_internal_mutable_debug_filename(), ::std::forward<Arg_>(value),
      args... );
  SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  // @@protoc_insertion_point(field_add:bloaty.Options.debug_filename)
}
inline const ::google::protobuf::RepeatedPtrField<::std::string>& Options::debug_filename()
    const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:bloaty.Options.debug_filename)
  return _internal_debug_filename();
}
inline ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL
Options::mutable_debug_filename() ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  // @@protoc_insertion_point(field_mutable_list:bloaty.Options.debug_filename)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_debug_filename();
}
inline const ::google::protobuf::RepeatedPtrField<::std::string>&
Options::_internal_debug_filename() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.debug_filename_;
}
inline ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL
Options::_internal_mutable_debug_filename() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.debug_filename_;
}

// repeated string data_source = 3;
inline int Options::_internal_data_source_size() const {
  return _internal_data_source().size();
}
inline int Options::data_source_size() const {
  return _internal_data_source_size();
}
inline void Options::clear_data_source() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.data_source_.Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
}
inline ::std::string* PROTOBUF_NONNULL Options::add_data_source()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::std::string* _s =
      _internal_mutable_data_source()->InternalAddWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), GetArena());
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  // @@protoc_insertion_point(field_add_mutable:bloaty.Options.data_source)
  return _s;
}
inline const ::std::string& Options::data_source(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:bloaty.Options.data_source)
  return _internal_data_source().Get(index);
}
inline ::std::string* PROTOBUF_NONNULL Options::mutable_data_source(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:bloaty.Options.data_source)
  return _internal_mutable_data_source()->Mutable(index);
}
template <typename Arg_, typename... Args_>
inline void Options::set_data_source(int index, Arg_&& value, Args_... args) {
  ::google::protobuf::internal::AssignToString(
      *_internal_mutable_data_source()->Mutable(index),
      ::std::forward<Arg_>(value), args... );
  // @@protoc_insertion_point(field_set:bloaty.Options.data_source)
}
template <typename Arg_, typename... Args_>
inline void Options::add_data_source(Arg_&& value, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::google::protobuf::internal::AddToRepeatedPtrField(
      ::google::protobuf::MessageLite::internal_visibility(), GetArena(),
      *_internal_mutable_data_source(), ::std::forward<Arg_>(value),
      args... );
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  // @@protoc_insertion_point(field_add:bloaty.Options.data_source)
}
inline const ::google::protobuf::RepeatedPtrField<::std::string>& Options::data_source()
    const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:bloaty.Options.data_source)
  return _internal_data_source();
}
inline ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL
Options::mutable_data_source() ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  // @@protoc_insertion_point(field_mutable_list:bloaty.Options.data_source)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_data_source();
}
inline const ::google::protobuf::RepeatedPtrField<::std::string>&
Options::_internal_data_source() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.data_source_;
}
inline ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL
Options::_internal_mutable_data_source() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.data_source_;
}

// optional int64 max_rows_per_level = 4 [default = 20];
inline bool Options::has_max_rows_per_level() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00002000U);
  return value;
}
inline void Options::clear_max_rows_per_level() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.max_rows_per_level_ = ::int64_t{20};
  ClearHasBit(_impl_._has_bits_[0], 0x00002000U);
}
inline ::int64_t Options::max_rows_per_level() const {
  // @@protoc_insertion_point(field_get:bloaty.Options.max_rows_per_level)
  return _internal_max_rows_per_level();
}
inline void Options::set_max_rows_per_level(::int64_t value) {
  _internal_set_max_rows_per_level(value);
  SetHasBit(_impl_._has_bits_[0], 0x00002000U);
  // @@protoc_insertion_point(field_set:bloaty.Options.max_rows_per_level)
}
inline ::int64_t Options::_internal_max_rows_per_level() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.max_rows_per_level_;
}
inline void Options::_internal_set_max_rows_per_level(::int64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.max_rows_per_level_ = value;
}

// optional .bloaty.Options.Demangle demangle = 5 [default = DEMANGLE_SHORT];
inline bool Options::has_demangle() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000080U);
  return value;
}
inline void Options::clear_demangle() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.demangle_ = 0;
  ClearHasBit(_impl_._has_bits_[0], 0x00000080U);
}
inline ::bloaty::Options_Demangle Options::demangle() const {
  // @@protoc_insertion_point(field_get:bloaty.Options.demangle)
  return _internal_demangle();
}
inline void Options::set_demangle(::bloaty::Options_Demangle value) {
  _internal_set_demangle(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000080U);
  // @@protoc_insertion_point(field_set:bloaty.Options.demangle)
}
inline ::bloaty::Options_Demangle Options::_internal_demangle() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return static_cast<::bloaty::Options_Demangle>(_impl_.demangle_);
}
inline void Options::_internal_set_demangle(::bloaty::Options_Demangle value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);

                                          assert(::google::protobuf::internal::ValidateEnum(
                                              value, ::bloaty::Options_Demangle_internal_data_));
                                          _impl_.demangle_ = value;
}

// optional .bloaty.Options.SortBy sort_by = 6 [default = SORTBY_BOTH];
inline bool Options::has_sort_by() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000100U);
  return value;
}
inline void Options::clear_sort_by() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.sort_by_ = 0;
  ClearHasBit(_impl_._has_bits_[0], 0x00000100U);
}
inline ::bloaty::Options_SortBy Options::sort_by() const {
  // @@protoc_insertion_point(field_get:bloaty.Options.sort_by)
  return _internal_sort_by();
}
inline void Options::set_sort_by(::bloaty::Options_SortBy value) {
  _internal_set_sort_by(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000100U);
  // @@protoc_insertion_point(field_set:bloaty.Options.sort_by)
}
inline ::bloaty::Options_SortBy Options::_internal_sort_by() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return static_cast<::bloaty::Options_SortBy>(_impl_.sort_by_);
}
inline void Options::_internal_set_sort_by(::bloaty::Options_SortBy value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);

                                          assert(::google::protobuf::internal::ValidateEnum(
                                              value, ::bloaty::Options_SortBy_internal_data_));
                                          _impl_.sort_by_ = value;
}

// optional int32 verbose_level = 7;
inline bool Options::has_verbose_level() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000200U);
  return value;
}
inline void Options::clear_verbose_level() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.verbose_level_ = 0;
  ClearHasBit(_impl_._has_bits_[0], 0x00000200U);
}
inline ::int32_t Options::verbose_level() const {
  // @@protoc_insertion_point(field_get:bloaty.Options.verbose_level)
  return _internal_verbose_level();
}
inline void Options::set_verbose_level(::int32_t value) {
  _internal_set_verbose_level(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000200U);
  // @@protoc_insertion_point(field_set:bloaty.Options.verbose_level)
}
inline ::int32_t Options::_internal_verbose_level() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.verbose_level_;
}
inline void Options::_internal_set_verbose_level(::int32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.verbose_level_ = value;
}

// optional uint64 debug_vmaddr = 11;
inline bool Options::has_debug_vmaddr() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000800U);
  return value;
}
inline void Options::clear_debug_vmaddr() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.debug_vmaddr_ = ::uint64_t{0u};
  ClearHasBit(_impl_._has_bits_[0], 0x00000800U);
}
inline ::uint64_t Options::debug_vmaddr() const {
  // @@protoc_insertion_point(field_get:bloaty.Options.debug_vmaddr)
  return _internal_debug_vmaddr();
}
inline void Options::set_debug_vmaddr(::uint64_t value) {
  _internal_set_debug_vmaddr(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000800U);
  // @@protoc_insertion_point(field_set:bloaty.Options.debug_vmaddr)
}
inline ::uint64_t Options::_internal_debug_vmaddr() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.debug_vmaddr_;
}
inline void Options::_internal_set_debug_vmaddr(::uint64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.debug_vmaddr_ = value;
}

// optional uint64 debug_fileoff = 12;
inline bool Options::has_debug_fileoff() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00001000U);
  return value;
}
inline void Options::clear_debug_fileoff() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.debug_fileoff_ = ::uint64_t{0u};
  ClearHasBit(_impl_._has_bits_[0], 0x00001000U);
}
inline ::uint64_t Options::debug_fileoff() const {
  // @@protoc_insertion_point(field_get:bloaty.Options.debug_fileoff)
  return _internal_debug_fileoff();
}
inline void Options::set_debug_fileoff(::uint64_t value) {
  _internal_set_debug_fileoff(value);
  SetHasBit(_impl_._has_bits_[0], 0x00001000U);
  // @@protoc_insertion_point(field_set:bloaty.Options.debug_fileoff)
}
inline ::uint64_t Options::_internal_debug_fileoff() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.debug_fileoff_;
}
inline void Options::_internal_set_debug_fileoff(::uint64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.debug_fileoff_ = value;
}

// repeated .bloaty.CustomDataSource custom_data_source = 8;
inline int Options::_internal_custom_data_source_size() const {
  return _internal_custom_data_source().size();
}
inline int Options::custom_data_source_size() const {
  return _internal_custom_data_source_size();
}
inline void Options::clear_custom_data_source() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.custom_data_source_.Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
}
inline const ::bloaty::CustomDataSource& Options::custom_data_source(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:bloaty.Options.custom_data_source)
  return _internal_custom_data_source().Get(index);
}
inline ::bloaty::CustomDataSource* PROTOBUF_NONNULL Options::mutable_custom_data_source(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:bloaty.Options.custom_data_source)
  return _internal_mutable_custom_data_source()->Mutable(index);
}
inline ::bloaty::CustomDataSource* PROTOBUF_NONNULL Options::add_custom_data_source()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::bloaty::CustomDataSource* _add =
      _internal_mutable_custom_data_source()->InternalAddWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), GetArena());
  SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  // @@protoc_insertion_point(field_add:bloaty.Options.custom_data_source)
  return _add;
}
inline const ::google::protobuf::RepeatedPtrField<::bloaty::CustomDataSource>& Options::custom_data_source() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:bloaty.Options.custom_data_source)
  return _internal_custom_data_source();
}
inline ::google::protobuf::RepeatedPtrField<::bloaty::CustomDataSource>* PROTOBUF_NONNULL
Options::mutable_custom_data_source() ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  // @@protoc_insertion_point(field_mutable_list:bloaty.Options.custom_data_source)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_custom_data_source();
}
inline const ::google::protobuf::RepeatedPtrField<::bloaty::CustomDataSource>&
Options::_internal_custom_data_source() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.custom_data_source_;
}
inline ::google::protobuf::RepeatedPtrField<::bloaty::CustomDataSource>* PROTOBUF_NONNULL
Options::_internal_mutable_custom_data_source() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.custom_data_source_;
}

// optional string disassemble_function = 9;
inline bool Options::has_disassemble_function() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000020U);
  return value;
}
inline void Options::clear_disassemble_function() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.disassemble_function_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000020U);
}
inline const ::std::string& Options::disassemble_function() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:bloaty.Options.disassemble_function)
  return _internal_disassemble_function();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void Options::set_disassemble_function(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000020U);
  _impl_.disassemble_function_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:bloaty.Options.disassemble_function)
}
inline ::std::string* PROTOBUF_NONNULL Options::mutable_disassemble_function()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000020U);
  ::std::string* _s = _internal_mutable_disassemble_function();
  // @@protoc_insertion_point(field_mutable:bloaty.Options.disassemble_function)
  return _s;
}
inline const ::std::string& Options::_internal_disassemble_function() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.disassemble_function_.Get();
}
inline void Options::_internal_set_disassemble_function(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.disassemble_function_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL Options::_internal_mutable_disassemble_function() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.disassemble_function_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE Options::release_disassemble_function() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:bloaty.Options.disassemble_function)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000020U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000020U);
  auto* released = _impl_.disassemble_function_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.disassemble_function_.Set("", GetArena());
  }
  return released;
}
inline void Options::set_allocated_disassemble_function(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000020U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000020U);
  }
  _impl_.disassemble_function_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.disassemble_function_.IsDefault()) {
    _impl_.disassemble_function_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:bloaty.Options.disassemble_function)
}

// optional string source_filter = 13;
inline bool Options::has_source_filter() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000040U);
  return value;
}
inline void Options::clear_source_filter() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.source_filter_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000040U);
}
inline const ::std::string& Options::source_filter() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:bloaty.Options.source_filter)
  return _internal_source_filter();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void Options::set_source_filter(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000040U);
  _impl_.source_filter_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:bloaty.Options.source_filter)
}
inline ::std::string* PROTOBUF_NONNULL Options::mutable_source_filter()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000040U);
  ::std::string* _s = _internal_mutable_source_filter();
  // @@protoc_insertion_point(field_mutable:bloaty.Options.source_filter)
  return _s;
}
inline const ::std::string& Options::_internal_source_filter() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.source_filter_.Get();
}
inline void Options::_internal_set_source_filter(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.source_filter_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL Options::_internal_mutable_source_filter() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.source_filter_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE Options::release_source_filter() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:bloaty.Options.source_filter)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000040U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000040U);
  auto* released = _impl_.source_filter_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.source_filter_.Set("", GetArena());
  }
  return released;
}
inline void Options::set_allocated_source_filter(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000040U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000040U);
  }
  _impl_.source_filter_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.source_filter_.IsDefault()) {
    _impl_.source_filter_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:bloaty.Options.source_filter)
}

// optional bool dump_raw_map = 14;
inline bool Options::has_dump_raw_map() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000400U);
  return value;
}
inline void Options::clear_dump_raw_map() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.dump_raw_map_ = false;
  ClearHasBit(_impl_._has_bits_[0], 0x00000400U);
}
inline bool Options::dump_raw_map() const {
  // @@protoc_insertion_point(field_get:bloaty.Options.dump_raw_map)
  return _internal_dump_raw_map();
}
inline void Options::set_dump_raw_map(bool value) {
  _internal_set_dump_raw_map(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000400U);
  // @@protoc_insertion_point(field_set:bloaty.Options.dump_raw_map)
}
inline bool Options::_internal_dump_raw_map() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.dump_raw_map_;
}
inline void Options::_internal_set_dump_raw_map(bool value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.dump_raw_map_ = value;
}

// -------------------------------------------------------------------

// CustomDataSource

// optional string name = 1;
inline bool CustomDataSource::has_name() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000002U);
  return value;
}
inline void CustomDataSource::clear_name() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.name_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
inline const ::std::string& CustomDataSource::name() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:bloaty.CustomDataSource.name)
  return _internal_name();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void CustomDataSource::set_name(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  _impl_.name_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:bloaty.CustomDataSource.name)
}
inline ::std::string* PROTOBUF_NONNULL CustomDataSource::mutable_name()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::std::string* _s = _internal_mutable_name();
  // @@protoc_insertion_point(field_mutable:bloaty.CustomDataSource.name)
  return _s;
}
inline const ::std::string& CustomDataSource::_internal_name() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.name_.Get();
}
inline void CustomDataSource::_internal_set_name(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.name_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL CustomDataSource::_internal_mutable_name() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.name_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE CustomDataSource::release_name() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:bloaty.CustomDataSource.name)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000002U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  auto* released = _impl_.name_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.name_.Set("", GetArena());
  }
  return released;
}
inline void CustomDataSource::set_allocated_name(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }
  _impl_.name_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.name_.IsDefault()) {
    _impl_.name_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:bloaty.CustomDataSource.name)
}

// optional string base_data_source = 2;
inline bool CustomDataSource::has_base_data_source() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000004U);
  return value;
}
inline void CustomDataSource::clear_base_data_source() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.base_data_source_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
}
inline const ::std::string& CustomDataSource::base_data_source() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:bloaty.CustomDataSource.base_data_source)
  return _internal_base_data_source();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void CustomDataSource::set_base_data_source(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  _impl_.base_data_source_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:bloaty.CustomDataSource.base_data_source)
}
inline ::std::string* PROTOBUF_NONNULL CustomDataSource::mutable_base_data_source()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::std::string* _s = _internal_mutable_base_data_source();
  // @@protoc_insertion_point(field_mutable:bloaty.CustomDataSource.base_data_source)
  return _s;
}
inline const ::std::string& CustomDataSource::_internal_base_data_source() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.base_data_source_.Get();
}
inline void CustomDataSource::_internal_set_base_data_source(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.base_data_source_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL CustomDataSource::_internal_mutable_base_data_source() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.base_data_source_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE CustomDataSource::release_base_data_source() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:bloaty.CustomDataSource.base_data_source)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000004U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  auto* released = _impl_.base_data_source_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.base_data_source_.Set("", GetArena());
  }
  return released;
}
inline void CustomDataSource::set_allocated_base_data_source(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  }
  _impl_.base_data_source_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.base_data_source_.IsDefault()) {
    _impl_.base_data_source_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:bloaty.CustomDataSource.base_data_source)
}

// repeated .bloaty.Regex rewrite = 3;
inline int CustomDataSource::_internal_rewrite_size() const {
  return _internal_rewrite().size();
}
inline int CustomDataSource::rewrite_size() const {
  return _internal_rewrite_size();
}
inline void CustomDataSource::clear_rewrite() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.rewrite_.Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
inline const ::bloaty::Regex& CustomDataSource::rewrite(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:bloaty.CustomDataSource.rewrite)
  return _internal_rewrite().Get(index);
}
inline ::bloaty::Regex* PROTOBUF_NONNULL CustomDataSource::mutable_rewrite(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:bloaty.CustomDataSource.rewrite)
  return _internal_mutable_rewrite()->Mutable(index);
}
inline ::bloaty::Regex* PROTOBUF_NONNULL CustomDataSource::add_rewrite()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::bloaty::Regex* _add =
      _internal_mutable_rewrite()->InternalAddWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), GetArena());
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_add:bloaty.CustomDataSource.rewrite)
  return _add;
}
inline const ::google::protobuf::RepeatedPtrField<::bloaty::Regex>& CustomDataSource::rewrite() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:bloaty.CustomDataSource.rewrite)
  return _internal_rewrite();
}
inline ::google::protobuf::RepeatedPtrField<::bloaty::Regex>* PROTOBUF_NONNULL
CustomDataSource::mutable_rewrite() ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_mutable_list:bloaty.CustomDataSource.rewrite)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_rewrite();
}
inline const ::google::protobuf::RepeatedPtrField<::bloaty::Regex>&
CustomDataSource::_internal_rewrite() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.rewrite_;
}
inline ::google::protobuf::RepeatedPtrField<::bloaty::Regex>* PROTOBUF_NONNULL
CustomDataSource::_internal_mutable_rewrite() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.rewrite_;
}

// -------------------------------------------------------------------

// Regex

// optional string pattern = 1;
inline bool Regex::has_pattern() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000001U);
  return value;
}
inline void Regex::clear_pattern() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.pattern_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
inline const ::std::string& Regex::pattern() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:bloaty.Regex.pattern)
  return _internal_pattern();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void Regex::set_pattern(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  _impl_.pattern_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:bloaty.Regex.pattern)
}
inline ::std::string* PROTOBUF_NONNULL Regex::mutable_pattern()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::std::string* _s = _internal_mutable_pattern();
  // @@protoc_insertion_point(field_mutable:bloaty.Regex.pattern)
  return _s;
}
inline const ::std::string& Regex::_internal_pattern() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.pattern_.Get();
}
inline void Regex::_internal_set_pattern(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.pattern_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL Regex::_internal_mutable_pattern() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.pattern_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE Regex::release_pattern() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:bloaty.Regex.pattern)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000001U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  auto* released = _impl_.pattern_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.pattern_.Set("", GetArena());
  }
  return released;
}
inline void Regex::set_allocated_pattern(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  _impl_.pattern_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.pattern_.IsDefault()) {
    _impl_.pattern_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:bloaty.Regex.pattern)
}

// optional string replacement = 2;
inline bool Regex::has_replacement() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000002U);
  return value;
}
inline void Regex::clear_replacement() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.replacement_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
inline const ::std::string& Regex::replacement() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:bloaty.Regex.replacement)
  return _internal_replacement();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void Regex::set_replacement(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  _impl_.replacement_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:bloaty.Regex.replacement)
}
inline ::std::string* PROTOBUF_NONNULL Regex::mutable_replacement()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::std::string* _s = _internal_mutable_replacement();
  // @@protoc_insertion_point(field_mutable:bloaty.Regex.replacement)
  return _s;
}
inline const ::std::string& Regex::_internal_replacement() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.replacement_.Get();
}
inline void Regex::_internal_set_replacement(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.replacement_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL Regex::_internal_mutable_replacement() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.replacement_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE Regex::release_replacement() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:bloaty.Regex.replacement)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000002U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  auto* released = _impl_.replacement_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.replacement_.Set("", GetArena());
  }
  return released;
}
inline void Regex::set_allocated_replacement(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }
  _impl_.replacement_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.replacement_.IsDefault()) {
    _impl_.replacement_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:bloaty.Regex.replacement)
}

#ifdef __GNUC__
#pragma GCC diagnostic pop
#endif  // __GNUC__

// @@protoc_insertion_point(namespace_scope)
}  // namespace bloaty


namespace google {
namespace protobuf {

template <>
struct is_proto_enum<::bloaty::Options_Demangle> : std::true_type {};
template <>
inline const EnumDescriptor* PROTOBUF_NONNULL GetEnumDescriptor<::bloaty::Options_Demangle>() {
  return ::bloaty::Options_Demangle_descriptor();
}
template <>
struct is_proto_enum<::bloaty::Options_SortBy> : std::true_type {};
template <>
inline const EnumDescriptor* PROTOBUF_NONNULL GetEnumDescriptor<::bloaty::Options_SortBy>() {
  return ::bloaty::Options_SortBy_descriptor();
}

}  // namespace protobuf
}  // namespace google

// @@protoc_insertion_point(global_scope)

#include "google/protobuf/port_undef.inc"
// clang-format on

#endif  // bloaty_2eproto_2epb_2eh
